

read.config <- function(filepath) {
  config.lines <- readLines(filepath)

  tokens <- strsplit(config.lines, split="[[:blank:]]+")
  
  linenum <- 1
  stopifnot(tokens[[linenum]][1] == "noftetratrees")
  stopifnot(length(tokens[[linenum]]) == 2)
  noftetratrees <- as.integer(tokens[[linenum]][2])
  
  linenum <- 2
  stopifnot(tokens[[linenum]][1] == "nofgenes")
  stopifnot(length(tokens[[linenum]]) == 2)
  nofgenes <- as.integer(tokens[[linenum]][2])
  
  linenum <- 3
  stopifnot(tokens[[linenum]][1] == "genelength")
  stopifnot(length(tokens[[linenum]]) == 2)
  genelength <- as.integer(tokens[[linenum]][2])
  
  linenum <- 4
  stopifnot(tokens[[linenum]][1] == "mutationrate")
  stopifnot(length(tokens[[linenum]]) == 2)
  mutationrate <- as.double(tokens[[linenum]][2])
  
  linenum <- 5
  stopifnot(tokens[[linenum]][1] == "gtree_seed")
  stopifnot(length(tokens[[linenum]]) == 2)
  gtree.seed <- as.integer(tokens[[linenum]][2])
 
  linenum <- 6
  stopifnot(tokens[[linenum]][1] == "sequence_seed")
  stopifnot(length(tokens[[linenum]]) == 2)
  sequence.seed <- as.integer(tokens[[linenum]][2])

  linenum <- 7
  ditree <- read.ditree.config(tokens, linenum)
  linenum <- ditree$lastlinenum + 1
  
  tetratrees <- read.alltetratrees.config(tokens, linenum, noftetratrees)
  
  gtrees <- list(nof=nofgenes, seed=gtree.seed, mutationrate=mutationrate)
  seqs <- list(len=genelength, seed=sequence.seed)
  network <- list(noftetratrees=noftetratrees, 
       ditree=ditree, tetratrees=tetratrees)                        
  list(gtrees=gtrees, seqs=seqs, network=network)
  }
  
  
  
read.ditree.config <- function(tokens, startlinenum) {
  linenum  <- startlinenum
  stopifnot(length(tokens[[linenum]]) == 3)
  stopifnot(tokens[[linenum]][1] == "ditree")
  stopifnot(tokens[[linenum]][2] == "ntips")
  ntips <- as.integer(tokens[[linenum]][3])
  nodes <- as.list(NULL)
  for (i in 1:(2*ntips-1)) {
    linenum <- startlinenum + i
    node <- read.node(tokens, linenum, (i <= ntips), (i == 2*ntips-1))
    nodes <- c(nodes, list(node))
    }
  list(nodes=nodes, lastlinenum=startlinenum+2*ntips-1)
  }
  

  
  
read.alltetratrees.config <- function(tokens, linenum, noftetratrees) {
  tetratrees <- NULL
  for (i in 1:noftetratrees) {
    tettree <- read.tetratree.config(tokens, linenum)
    tetratrees <- c(tetratrees, list(tettree))
    linenum <- tettree$lastlinenum + 1
    }
  tetratrees
  }
  
  
  
read.tetratree.config <- function(tokens, startlinenum) {
  linenum  <- startlinenum
  stopifnot(length(tokens[[linenum]]) == 3)
  stopifnot(tokens[[linenum]][1] == "tetratree")
  stopifnot(tokens[[linenum]][2] == "ntips")
  ntips <- as.integer(tokens[[linenum]][3])
  nodes <- as.list(NULL)
  for (i in 1:(2*ntips-1)) {
    linenum <- startlinenum + i
    node <- read.node(tokens, linenum, (i <= ntips), (i == 2*ntips-1))
    nodes <- c(nodes, list(node))
    }
  linenum <- startlinenum + 2*ntips
  stopifnot(length(tokens[[linenum]]) == 14)
  stopifnot(tokens[[linenum]][1] == "hyb")
  stopifnot(tokens[[linenum]][4] == "join")
  stopifnot(tokens[[linenum]][7] == "foot1")
  stopifnot(tokens[[linenum]][11] == "foot2")
  hybheight <- -1
  hybpop <- -1
  foot1clade <- ""
  foot1height <- -1
  foot1pop <- -1
  joinheight <- -1
  joinpop <- -1
  foot2clade <- ""
  foot2height <- -1
  foot2pop <- -1
  hybheight <- as.double(tokens[[linenum]][2])
  hybpop <- as.double(tokens[[linenum]][3])
  if (tokens[[linenum]][5] != "*") { join = TRUE } else { join = FALSE }
  foot1clade <- paste(tokens[[linenum]][8], "A", sep="")
  foot1height <- as.double(tokens[[linenum]][9])
  foot1pop <- as.double(tokens[[linenum]][10])
  if (join) {
    joinheight <- as.double(tokens[[linenum]][5])
    joinpop <- as.double(tokens[[linenum]][6])
    nodes[[length(nodes)]]$prevpop <- joinpop
    } else {
    foot2clade <- paste(tokens[[linenum]][12], "A", sep="")
    foot2height <- as.double(tokens[[linenum]][13])
    foot2pop <- as.double(tokens[[linenum]][14])
    }
  legs <- list(hybheight=hybheight, hybpop=hybpop,
               foot1clade=foot1clade, foot1height=foot1height, foot1pop=foot1pop,
               joinheight=joinheight, joinpop=joinpop,
               foot2clade=foot2clade, foot2height=foot2height, foot2pop=foot2pop)
  list(nodes=nodes, legs=legs, lastlinenum=startlinenum+2*ntips)
  }
  
  
read.node <- function(tokens, linenum, tip, root) {
  stopifnot(length(tokens[[linenum]]) == 7)
  stopifnot(tokens[[linenum]][4] == "tippop")
  stopifnot(tokens[[linenum]][6] == "prevpop")
  indivs <- -1
  height <- 0
  tippop <- -1
  prevpop <- -1
  clade <- tokens[[linenum]][1]
   
  if (root) {
    stopifnot(tokens[[linenum]][7] == "*")
    } else {
    prevpop <- as.double(tokens[[linenum]][7])
    }
  
  if (tip) {
    stopifnot(nchar(clade) == 1)
    stopifnot(tokens[[linenum]][2] == "indivs")
    indivs <- as.integer(tokens[[linenum]][3])
    tippop <- as.double(tokens[[linenum]][5])
    } else {
    stopifnot(nchar(clade) >= 5)
    stopifnot(tokens[[linenum]][2] == "height")
    height <- as.double(tokens[[linenum]][3])
    }
    
  list(clade=clade, indivs=indivs, height=height,
               tippop=tippop, prevpop=prevpop, root=root)  
  } 





# From http://www.random.org/integers/
# in range 1-1e9, sorted
gtreeseeds <- c(
16221998, 20436219, 196180401, 210844997, 211439472, 
306922403, 311282050, 317222442, 348309956, 356269750,
364259989, 416078521, 421634045, 442914084, 446471823,
446807089, 450913054, 452480665, 504817679, 509080660,
522083444, 547644507, 548404454, 559105341, 565446238,
567709881, 620311867, 620902707, 627516603, 647992366,
659934868, 666191630, 690177136, 714321647, 715859060,
725744643, 730792087, 732164171, 767287320, 771819050,
804500367, 825668979, 863173150, 876291813, 943013481,
945925845, 946947416, 948237106, 971413945, 987257393)



# From http://www.random.org/integers/
# in range 1-1e9, sorted
seqseeds <- c(
64790, 7802451, 9576782, 9853658, 28745232,
84641065, 149741783, 163040601, 171711909, 176042877,
185448362, 228090536, 252577395, 253762625, 283110154,
323368843, 391148348, 392305196, 404877985, 418265283,
450595930, 465562755, 469039815, 539467750, 541379297,
567851335, 577997312, 578150656, 594300654, 636932155,
657181079, 691461386, 710799019, 721090926, 760206797,
782625416, 818433907, 838290114, 842182274, 848544304,
859617877, 862624605, 873764711, 899129444, 920077511,
927536929, 931108446, 936450194, 944806350, 965189343
)




